<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;

/**
 * @OA\Schema(
 *   title="Game",
 *   type="object",
 *   @OA\Property(type="integer", property="id"),
 *   @OA\Property(type="integer", property="server_id"),
 *   @OA\Property(type="number", property="elo"),
 *   @OA\Property(type="string", property="created_at", format="date-time"),
 *   @OA\Property(type="string", property="updated_at", format="date-time"),
 * )
 */
class Game extends Model
{
    use HasFactory;

    protected static function booted(): void
    {
        /*
        static::addGlobalScope('finished', function (Builder $builder) {
        $builder->where("state", "finished");
        });
        */
    }

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'server_id',
        'mode',
    ];

    protected $hidden = [
        'state'
    ];

    public function users()
    {
        return $this->belongsToMany(User::class, 'users_games', 'game_id', 'user_id')->using(UserGame::class)->withPivot("team_id", "rank", "elo");
    }

    public function server(): BelongsTo
    {
        return $this->belongsTo(Server::class, 'server_id');
    }

    public function start()
    {
        $this->started_at = Carbon::now();
        $this->state = 'playing';
        $this->save();
    }

    private function calculate_ranks($user_data)
    {
        // We first do the initial sorting to remove all oddities
        // that the server could have added
        // (we sort in reverse because the server sorts the leaderboard in reverse)
        usort($user_data, function ($a, $b) {
            return $b["rank"] <=> $a["rank"];
        });

        // We set the rank to a value between [1;4] and add the team id
        $sorted_user_data = [];
        foreach ($user_data as $k => $user) {
            $u = $this->users()->findOrFail($user["id"]);
            $sorted_user_data[] = [
                "id" => $user["id"],
                "rank" => $k + 1,
                "team_id" => $u->pivot->team_id,
                "elo" => $u->elo,
                "game_elo" => 0
            ];
        }
        $user_data = $sorted_user_data;

        if (strcmp($this->mode, "team") == 0) {
            $winning_team = $user_data[0]["team_id"];
            // We sort again but this time applying team logic
            // We cut the players in two sets, wining team and losing team
            // We sort woth sets and add them back together
            // Exemple: (id, team_id, rank): (1, 0, 1), (2, 1, 2), (3, 1, 3), (4, 0, 4)
            // Will be sorted (1, 0, 1), (4, 0, 4), (2, 1, 2), (3, 1, 3)
            usort($user_data, function ($a, $b) use ($winning_team) {
                if ($a["team_id"] == $winning_team && $b["team_id"] == $winning_team) {
                    return $a["rank"] <=> $b["rank"];
                } else if ($a["team_id"] == $winning_team && $b["team_id"] != $winning_team) {
                    return -1;
                } else if ($a["team_id"] != $winning_team && $b["team_id"] == $winning_team) {
                    return 1;
                }
                return $a["rank"] <=> $b["rank"];
            });

            // We then have to re-apply the rank-setting function to set the ranks in stone
            $sorted_user_data = [];
            foreach ($user_data as $k => $user) {
                $u = $this->users()->findOrFail($user["id"]);
                $user["rank"] = $k + 1;
                $sorted_user_data[] = $user;
            }
            $user_data = $sorted_user_data;
        }

        return $user_data;
    }

    private function probability_function($users, $user, $D, $N)
    {
        // We first calculate the probabilit of winning for the player.
        // For player A agains all players i, R_x being the current ELO of x
        // and N the number of players in a game, we have
        // E_A = { sum from {1 <= i <= N, i <> A} {1 over {1 + 10^{(R_i-R_a) / D}}}} over { {N cdot (N - 1)} over 2 }

        $E_A = 0;
        foreach($users as $other) {
            if ($user == $other)
                continue;
            
            $E_A += 1 / (1 + 10**(($other["elo"] - $user["elo"]) / $D));
        }

        return $E_A / ($N * ($N - 1) / 2);
    }

    private function score_function($user, $alpha, $N) {
        // This function calculates the gain from the rank of the player
        // S^exp_A ( p_A, %alpha ) = { a^{N - p_A} - 1 } over { sum from i = 1 to N {a^{N-i} - 1} }; %alpha in [ 0;+infinity ]

        $top = ($alpha ** ($N - $user["rank"]) - 1);

        $bottom = 0;
        for($i = 1; $i <= $N; $i++) {
            $bottom += $alpha ** ($N - $i) - 1;
        }

        return $top / $bottom;
    }

    private function calculate_elo($user_data)
    {
        // Taken from https://towardsdatascience.com/developing-a-generalized-elo-rating-system-for-multiplayer-games-b9b495e87802
        $D = 400;
        $K = 32;
        $alpha = 2;
        $N = count($user_data);

        // We first calculate the probabilit of winning for each players.
        // For player A agains all players i, R_x being the current ELO of x
        // and N the number of players in a game, we have
        // E_A = { sum from {1 <= i <= N, i <> A} {1 over {1 + 10^{(R_i-R_a) / D}}}} over { {N cdot (N - 1)} over 2 }

        foreach($user_data as &$user)
        {
            $user["E_A"] = $this->probability_function($user_data, $user, $D, $N);
            $user["new_elo"] = $user["elo"] + $K * ($N - 1) * ($this->score_function($user, $alpha, $N) - $user["E_A"]);
            $user["game_elo"] = $user["new_elo"] - $user["elo"];
        }
        return $user_data;
    }

    public function end($user_data)
    {
        $user_data = $this->calculate_ranks($user_data);

        if ($this->server->official)
        {
            $user_data = $this->calculate_elo($user_data);
        }

        foreach ($user_data as $user) {
            $this->users()->updateExistingPivot($user['id'], ["rank" => $user["rank"], "elo" => $user["game_elo"]]);
            $u = User::find($user["id"]);
            if ($u !== null) {
                $u->elo = $user["new_elo"];
                $u->save();
            }
        }

        $this->finished_at = Carbon::now();
        $this->state = 'finished';
        $this->push();
    }
}