<?php

namespace Tests\Feature;

use App\Models\Server;
use App\Models\User;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Tests\TestCase;

class ConnectTest extends TestCase
{
    use RefreshDatabase;

    public function test_full_good() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);
        $server->online = true;
        $server->save();

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "ip",
            "port"
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "user" => [
                "id",
                "username",
                "created_at",
                "updated_at"
            ]
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/disconnect');
        $response->assertStatus(204);
    }

    public function test_connect_already_connected() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);
        $server->online = true;
        $server->save();

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "ip",
            "port"
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "user" => [
                "id",
                "username",
                "created_at",
                "updated_at"
            ]
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server->id . '/connect', [
            'token' => '0705183cee5b5e7bbe0056ca9637011e095abde3b8b4968e0d03939c78700b8a'
        ]);
        $response->assertStatus(403);
    }

    public function test_connect_offline_server() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);
        $server->online = false;
        $server->save();

        $user_token = Auth::guard('client')->login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(403);
    }

    public function test_accept_not_connecting() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);
        $server->online = true;
        $server->save();

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);
        
        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(403);
    }

    public function test_accept_wrong_token() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);
        $server->online = true;
        $server->save();

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "ip",
            "port"
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => '0705183cee5b5e7bbe0056ca9637011e095abde3b8b4968e0d03939c78700b8a'
        ]);
        $response->assertStatus(403);
    }

    public function test_accept_wrong_server() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server2 = Server::create([
            'login' => 'test2',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);
        $server2->online = true;
        $server2->save();

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);
        $server->online = true;
        $server->save();

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server2->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "ip",
            "port"
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(403);
    }

    public function test_dicsonnect_wrong_server() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server2 = Server::create([
            'login' => 'test2',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);
        $server2->online = true;
        $server2->save();

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);
        $server->online = true;
        $server->save();

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server2->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "ip",
            "port"
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/disconnect');
        $response->assertStatus(403);
    }

    public function test_good_heartbeat()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);

        $user_token = Auth::guard('client')->login($user);
        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/auth/server/heartbeat', [
            'ip' => '127.0.0.1',
            'port' => 12345,
            'pubkey' => '-----BEGIN PUBLIC KEY-----
MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAy7IwoHhmFbuVp1vsu9IK
DLenHdy7IPJxMaw2Xq7zbgTZXlYQjNFky9coiyYWMXw2i7g1f3xptKvIKbDuEMvG
ifCwXVRAOHJKehhwVVL7IRRjleb3YGm5m1lLqPdYvfecn20mFdwuk/6Rkg003fKs
JsZiLCgrRknlQbIatiTakBGMu+D3pCwHc1MLEitEVUPBECLM66DBVKo5o1/vWQZF
glfcOB7nQpzyRagFUuRnPhYDSFNma8hkYGQctHpgdFUFq7Oofj8L3wxLM77h2H9s
ib5ctqtHzWneJ9APCB0YQ7dI4enzpeFi16x9/tuJS6jOwt5Is9PEVRtSrw9wKQbP
hvYbZGcPm7okV9ut3c/iJgtqta6dabIXkVTbHXrPtSW8Rs5toykj6DNAcCrvTEgP
dFmzbYJJv6Yfo219MVvEtLY0cC43gwXNMoxykdxBzCY0bihiRefaFK1g+6NR5H4H
KYRcc3SMB3YSkGNnzlSw89WCVvdnlU3hpcK9/M3hSvPsqUBB/uYNG5CMvRTLg1uO
q7BbLlbzqXh6utyQ/92QqYYrRwgQdRTamXRdcrCf3Ni4PZEem03/bfzBbN946Rgp
2wusu+zAWwwPbbigXnzC9VinPw8MLW54NzdnR0cp8b6HToi6Jf9IByb2O/Jf9LOv
RVQXdit0Wzv68t7iQ86vVyMCAwEAAQ==
-----END PUBLIC KEY-----',
            'team' => false
        ]);
        $response->assertStatus(204);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $user_token
        ])->post('/api/server/' . $server->id . '/connect', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "ip",
            "port",
            "pubkey"
        ]);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/user/' . $user->id . '/accept', [
            'token' => 'aef773374b964c42cbb44bf11452b8a912737ad3175341d1694591ef58a19ce4'
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "user" => [
                "id",
                "username",
                "created_at",
                "updated_at"
            ]
        ]);
    }
}
