<?php

namespace Tests\Feature;

use App\Models\Game;
use App\Models\Server;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Tests\TestCase;

class HeartbeatTest extends TestCase
{
    use RefreshDatabase;

    public function test_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);

        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/auth/server/heartbeat', [
                'ip' => '127.0.0.1',
                'port' => 12345,
                'pubkey' => '-----BEGIN PUBLIC KEY-----
MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAy7IwoHhmFbuVp1vsu9IK
DLenHdy7IPJxMaw2Xq7zbgTZXlYQjNFky9coiyYWMXw2i7g1f3xptKvIKbDuEMvG
ifCwXVRAOHJKehhwVVL7IRRjleb3YGm5m1lLqPdYvfecn20mFdwuk/6Rkg003fKs
JsZiLCgrRknlQbIatiTakBGMu+D3pCwHc1MLEitEVUPBECLM66DBVKo5o1/vWQZF
glfcOB7nQpzyRagFUuRnPhYDSFNma8hkYGQctHpgdFUFq7Oofj8L3wxLM77h2H9s
ib5ctqtHzWneJ9APCB0YQ7dI4enzpeFi16x9/tuJS6jOwt5Is9PEVRtSrw9wKQbP
hvYbZGcPm7okV9ut3c/iJgtqta6dabIXkVTbHXrPtSW8Rs5toykj6DNAcCrvTEgP
dFmzbYJJv6Yfo219MVvEtLY0cC43gwXNMoxykdxBzCY0bihiRefaFK1g+6NR5H4H
KYRcc3SMB3YSkGNnzlSw89WCVvdnlU3hpcK9/M3hSvPsqUBB/uYNG5CMvRTLg1uO
q7BbLlbzqXh6utyQ/92QqYYrRwgQdRTamXRdcrCf3Ni4PZEem03/bfzBbN946Rgp
2wusu+zAWwwPbbigXnzC9VinPw8MLW54NzdnR0cp8b6HToi6Jf9IByb2O/Jf9LOv
RVQXdit0Wzv68t7iQ86vVyMCAwEAAQ==
-----END PUBLIC KEY-----',
                'team' => false
            ]);
        $response->assertStatus(204);
    }

    public function test_no_data()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);

        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/auth/server/heartbeat', [
            ]);
        $response->assertStatus(422);
    }

    public function test_malformed_key()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);

        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/auth/server/heartbeat', [
                'ip' => '127.0.0.1',
                'port' => 12345,
                'pubkey' => 'Lorem ipsum dolor sit amet',
                'team' => false
            ]);
        $response->assertStatus(422);
    }

    public function test_change_pubkey()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $user2 = User::create([
            'username' => 'test2',
            'email' => 'test2@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
        ]);

        $user->server_id = $server->id;
        $user->server_state = 'connected';
        $user->server_token = null;
        $user->save();

        $user2->server_id = $server->id;
        $user2->server_state = 'connected';
        $user2->server_token = null;
        $user2->save();

        $server->heartbeat('127.0.0.1', 12345, 'pubkey', false);
        $server->last_heartbeat_at = Carbon::now()->subMinutes(5);
        $server->save();

        $game = Game::create([
            "server_id" => $server->id,
            "mode" => "ffa"
        ]);

        $game->users()->attach($user->id);
        $game->users()->attach($user2->id);

        $server = $server->refresh();
        $this->assertEquals(true, $server->online);
        $this->assertEquals("127.0.0.1", $server->ip);
        $this->assertEquals(12345, $server->port);
        $this->assertEquals("pubkey", $server->pubkey);

        $server_token = Auth::guard('server')->login($server);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $server_token
        ])->post('/api/auth/server/heartbeat', [
                'ip' => '127.0.0.1',
                'port' => 12345,
                'pubkey' => '-----BEGIN PUBLIC KEY-----
MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAy7IwoHhmFbuVp1vsu9IK
DLenHdy7IPJxMaw2Xq7zbgTZXlYQjNFky9coiyYWMXw2i7g1f3xptKvIKbDuEMvG
ifCwXVRAOHJKehhwVVL7IRRjleb3YGm5m1lLqPdYvfecn20mFdwuk/6Rkg003fKs
JsZiLCgrRknlQbIatiTakBGMu+D3pCwHc1MLEitEVUPBECLM66DBVKo5o1/vWQZF
glfcOB7nQpzyRagFUuRnPhYDSFNma8hkYGQctHpgdFUFq7Oofj8L3wxLM77h2H9s
ib5ctqtHzWneJ9APCB0YQ7dI4enzpeFi16x9/tuJS6jOwt5Is9PEVRtSrw9wKQbP
hvYbZGcPm7okV9ut3c/iJgtqta6dabIXkVTbHXrPtSW8Rs5toykj6DNAcCrvTEgP
dFmzbYJJv6Yfo219MVvEtLY0cC43gwXNMoxykdxBzCY0bihiRefaFK1g+6NR5H4H
KYRcc3SMB3YSkGNnzlSw89WCVvdnlU3hpcK9/M3hSvPsqUBB/uYNG5CMvRTLg1uO
q7BbLlbzqXh6utyQ/92QqYYrRwgQdRTamXRdcrCf3Ni4PZEem03/bfzBbN946Rgp
2wusu+zAWwwPbbigXnzC9VinPw8MLW54NzdnR0cp8b6HToi6Jf9IByb2O/Jf9LOv
RVQXdit0Wzv68t7iQ86vVyMCAwEAAQ==
-----END PUBLIC KEY-----',
                'team' => false
        ]);
        $response->assertStatus(204);

        $server = $server->refresh();
        $this->assertEquals(true, $server->online);
        $this->assertEquals('127.0.0.1', $server->ip);
        $this->assertEquals(12345, $server->port);
        $this->assertEquals('-----BEGIN PUBLIC KEY-----
MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAy7IwoHhmFbuVp1vsu9IK
DLenHdy7IPJxMaw2Xq7zbgTZXlYQjNFky9coiyYWMXw2i7g1f3xptKvIKbDuEMvG
ifCwXVRAOHJKehhwVVL7IRRjleb3YGm5m1lLqPdYvfecn20mFdwuk/6Rkg003fKs
JsZiLCgrRknlQbIatiTakBGMu+D3pCwHc1MLEitEVUPBECLM66DBVKo5o1/vWQZF
glfcOB7nQpzyRagFUuRnPhYDSFNma8hkYGQctHpgdFUFq7Oofj8L3wxLM77h2H9s
ib5ctqtHzWneJ9APCB0YQ7dI4enzpeFi16x9/tuJS6jOwt5Is9PEVRtSrw9wKQbP
hvYbZGcPm7okV9ut3c/iJgtqta6dabIXkVTbHXrPtSW8Rs5toykj6DNAcCrvTEgP
dFmzbYJJv6Yfo219MVvEtLY0cC43gwXNMoxykdxBzCY0bihiRefaFK1g+6NR5H4H
KYRcc3SMB3YSkGNnzlSw89WCVvdnlU3hpcK9/M3hSvPsqUBB/uYNG5CMvRTLg1uO
q7BbLlbzqXh6utyQ/92QqYYrRwgQdRTamXRdcrCf3Ni4PZEem03/bfzBbN946Rgp
2wusu+zAWwwPbbigXnzC9VinPw8MLW54NzdnR0cp8b6HToi6Jf9IByb2O/Jf9LOv
RVQXdit0Wzv68t7iQ86vVyMCAwEAAQ==
-----END PUBLIC KEY-----', $server->pubkey);

        $user = $user->refresh();
        $this->assertEquals(null, $user->server_id);
        $this->assertEquals("offline", $user->server_state);
        $this->assertEquals(null, $user->server_token);
        $user2 = $user2->refresh();
        $this->assertEquals(null, $user2->server_id);
        $this->assertEquals("offline", $user2->server_state);
        $this->assertEquals(null, $user2->server_token);

        $this->assertDatabaseCount("games", 0);
        $this->assertDatabaseCount("users_games", 0);
    }
}